@@ -138,7 +138,7 @@ pkg_install = function(pkgs, install = TRUE, ...) {
   if (isTRUE(install)) install = getOption(
     'xfun.install.packages',
     if (is.na(Sys.getenv('RENV_PROJECT', NA)) || !loadable('renv')) install.packages else {
-      function(pkgs, lib = NULL, ...) renv::install(pkgs, library = lib, ...)
+      function(pkgs, lib = NULL, ...) getFromNamespace('install', 'renv')(pkgs, library = lib, ...)
     }
   )
   if (identical(install, 'pak')) install = pak::pkg_install
