@@ -16,6 +16,12 @@
 #'   arguments are `list(units = 'in', onefile = FALSE, width = 7, height = 7,
 #'   res = 96)`. If any of these arguments is not present in the device
 #'   function, it will be dropped.
+#' @param dev.keep Indices of plots to be kept. The indices can be either
+#'   numeric (positive or negative integers) or logical. Negative integers and
+#'   false values will remove the corresponding plots. For example, if the code
+#'   generated 3 plots, `dev.keep = c(1, 3)`, `-2`, and `c(T, F, T)` are
+#'   equivalent ways to remove the second plot. The special value `dev.keep =
+#'   'last'` means to keep the last plot only.
 #' @param message,warning,error If `TRUE`, record and store messages / warnings
 #'   / errors in the output. If `FALSE`, suppress them. If `NA`, do not process
 #'   them (messages will be emitted to the console, and errors will halt the
@@ -55,8 +61,8 @@
 #' file.remove(unlist(plots))
 record = function(
   code = NULL, dev = 'png', dev.path = 'xfun-record', dev.ext = dev_ext(dev),
-  dev.args = list(), message = TRUE, warning = TRUE, error = NA, cache = list(),
-  print = record_print, print.args = list(),
+  dev.args = list(), dev.keep = TRUE, message = TRUE, warning = TRUE, error = NA,
+  cache = list(), print = record_print, print.args = list(),
   verbose = getOption('xfun.record.verbose', 0), envir = parent.frame()
 ) {
   new_result = function(x = list()) structure(x, class = 'xfun_record_results')
@@ -160,6 +166,11 @@ record = function(
       old_files <<- files
       old_times <<- file.mtime(files)
       if ((n <- length(plots)) == 0) return()
+      # if an empty plot is recorded, remove it (e.g., yihui/litedown#96)
+      if (n == 1 && empty_plot(old_plot)) {
+        file.remove(plots); old_files <<- character(); old_times <<- NULL
+        return()
+      }
 
       # indices of plots in results
       i = which(vapply(res, inherits, logical(1), 'record_plot'))
@@ -190,16 +201,22 @@ record = function(
   codes = handle(split_source(code, merge_comments = TRUE, line_number = TRUE))
   # code may contain syntax errors
   if (is_error(codes)) {
-    add_result(code, 'source'); add_result(attr(codes, 'condition')$message, 'error')
+    add_result(code, 'source'); add_result(attr2(codes, 'condition')$message, 'error')
     return(new_result(res))
   }
 
+  # trim error message and remove the uninformative part (yihui/litedown#109)
+  trim_error = function(e) {
+    x = sub('\\s+$', '', as.character(e))
+    sub(' in eval\\(expr, envir(, enclos)?\\):', ':', x)
+  }
   handle_message = function(type, add = TRUE) {
-    mf = sub('^(.)', 'muffle\\U\\1', type, perl = TRUE)
+    err = type == 'error'
+    mf = if (!err) sub('^(.)', 'muffle\\U\\1', type, perl = TRUE)
     function(e) {
       if (is.na(add)) return()
-      if (isTRUE(add)) add_result(e$message, type)
-      if (type %in% c('message', 'warning')) invokeRestart(mf)
+      if (isTRUE(add)) add_result(if (err) trim_error(e) else e$message, type)
+      if (!err) invokeRestart(mf)
     }
   }
   handle_m = handle_message('message', message)
@@ -219,14 +236,17 @@ record = function(
     out = NULL
     con = textConnection('out', 'w', local = TRUE)
     on.exit(close(con))
-    sink(con); on.exit({ sink(); close(con) })
+    # capture try() messages
+    opts = if (is.null(getOption('try.outFile'))) options(try.outFile = con)
+    sink(con); on.exit({ sink(); close(con); options(opts) })
     expr  # lazy evaluation
     on.exit()  # if no error occurred, clear up previous on-exit calls
-    sink()
-    close(con)
+    sink(); close(con); options(opts)
     if (length(out)) add_result(out, 'output')
     expr
   }
+  # disable ANSI sequences in text output
+  opts = options(cli.num_colors = 1); on.exit(options(opts), add = TRUE)
   n = length(codes)
   for (i in seq_len(n)) {
     add_result(code <- codes[[i]], 'source')
@@ -241,10 +261,13 @@ record = function(
     }
     # print value (via record_print()) if visible
     if (!is_error(out) && out$visible) {
+      val = out$value
+      # if val is data.table, check if it needs to be printed
+      if (inherits(val, 'data.table') && 'data.table' %in% loadedNamespaces() &&
+          !data.table::shouldPrint(val)) next
       if (is.null(print)) print = record_print
       if (!is.function(print)) print = record_print.default
       p_args = print.args
-      val = out$value
       # index print args by first class of out unless args are wrapped in I()
       if (!inherits(p_args, 'AsIs')) p_args = p_args[[class(val)[1]]]
       out = handle_eval(if (length(p_args) == 0) print(val) else do.call(
@@ -260,6 +283,22 @@ record = function(
   dev_off()
   handle_plot(TRUE)
 
+  # filter plots
+  if (!isTRUE(dev.keep)) {
+    p = get_plots()  # all plots
+    if (identical(dev.keep, 'last')) dev.keep = length(p)
+    d = setdiff(p, p[dev.keep])  # plots to be deleted
+    if (length(d)) {
+      for (i in seq_along(res)) {
+        if (!inherits(r <- res[[i]], 'record_plot') || all(is.na(k <- match(d, r)))) next
+        r2 = r[-k[!is.na(k)]]  # remove plots matching d
+        attributes(r2) = attributes(r)  # preserve class
+        res[[i]] = r2
+      }
+      file.remove(d)
+    }
+  }
+
   # remove empty blocks
   res = Filter(length, res)
   res = merge_record(res)
@@ -349,6 +388,26 @@ dev_ext = function(dev) {
   file_ext(name)
 }
 
+# test if a recorded plot is empty
+empty_plot = function(p) {
+  xs = lapply(p[[1]], function(x) x[[2]][[1]])
+  for (x in xs) {
+    if ('name' %in% names(x)) {
+      # base graphics
+      if (!x$name %in% empty_calls) return(FALSE)
+    } else if (is.call(x)) {
+      # grid graphics
+      if (!identical(as.character(x[[1]]), 'requireNamespace')) return(FALSE)
+    }
+  }
+  TRUE
+}
+
+empty_calls = c(
+  'C_clip', 'C_layout', 'C_par', 'C_plot_window', 'C_strHeight', 'C_strWidth',
+  'palette', 'palette2'
+)
+
 is_error = function(x) inherits(x, 'try-error')
 
 #' @param to The output format (text, markdown, or html).
@@ -375,7 +434,7 @@ format.xfun_record_results = function(
         if (is_md && cls == 'plot') {
           z = sprintf('![%s](<%s>)', alt, z)
         } else {
-          o = attr(z, 'opts'); a = o$attr
+          o = attr2(z, 'opts'); a = o$attr
           z = gsub('^(\\s*\n)+|\n\\s*$', '', one_string(z))  # trim blank lines
           if (cls != 'source') z = paste0(o$comment %||% '#> ', split_lines(z))
           if (is_md) {
@@ -408,7 +467,7 @@ format.xfun_record_results = function(
       paste0(
         sprintf(
           '<pre class="%s"><code>',
-          if (cls == 'source') paste0('language-r" data-start="', attr(z, 'lines')[1]) else cls
+          if (cls == 'source') paste0('language-r" data-start="', attr2(z, 'lines')[1]) else cls
         ),
         html_escape(one_string(z)), '</code></pre>'
       )
